@@ -8,7 +8,8 @@ import subprocess  # nosec: B404
 from dataclasses import asdict, dataclass
 from typing import List, Optional, Tuple
 
-from opendapi.defs import ALL_OPENDAPI_SUFFIXES, REFS_PREFIXES
+from opendapi.cli.common import print_encoded_breadcrumbs
+from opendapi.defs import ALL_OPENDAPI_SUFFIXES, REFS_PREFIXES, CommitType
 from opendapi.logging import logger
 
 
@@ -20,7 +21,8 @@ def run_git_command(cwd: str, command_split: List[str]) -> str:
             cwd=cwd,
         )  # nosec
     except subprocess.CalledProcessError as exc:
-        # TODO: figure out why this is not sent to sentry but CI fails
+        # KBtodo: Figure out why this is not sent to sentry but CI fails
+        print_encoded_breadcrumbs(f"git command {command_split}: {exc}")
         raise RuntimeError(f"git command {command_split}: {exc}") from exc
 
 
@@ -50,7 +52,7 @@ def get_merge_base(cwd: str, current_ref: str, base_ref: str) -> str:
         .decode("utf-8")
         .strip()
     )
-    logger.info(f"Merge base of {current_ref} and {base_ref} is {merge_base}")
+    logger.info("Merge base of %s and %s is %s", current_ref, base_ref, merge_base)
     return merge_base
 
 
@@ -67,7 +69,7 @@ def get_upstream_commit_sha(cwd: str, ref: str, steps: int) -> str:
         .decode("utf-8")
         .strip()
     )
-    logger.info(f"Upstream commit SHA of {ref}~{steps} is {upstream_sha}")
+    logger.info("Upstream commit SHA of %s~%s is %s", ref, steps, upstream_sha)
     return upstream_sha
 
 
@@ -166,7 +168,7 @@ def add_untracked_opendapi_files(
         all_files = [r.split(" ", 2)[-1] for r in result.split("\n") if r]
         run_git_command(cwd, ["git", "add", *all_files])
         return len(all_files)
-    return 0
+    return 0  # pragma: no cover
 
 
 def get_git_diff_filenames(
@@ -265,7 +267,7 @@ class GitCommitStasher:
             self._reset()
 
 
-@dataclass
+@dataclass(frozen=True)
 class ChangeTriggerEvent:
     """
     Summary information for git changes
@@ -287,6 +289,7 @@ class ChangeTriggerEvent:
     repository: Optional[str] = None
     repo_full_name: Optional[str] = None
     pull_request_link: Optional[str] = None
+    original_pr_author: Optional[str] = None
 
     def __post_init__(self):
         """Post init checks"""
@@ -363,3 +366,16 @@ class ChangeTriggerEvent:
     def as_dict(self) -> dict:
         """Get the event as a dictionary"""
         return asdict(self)
+
+    def commit_type_to_sha(
+        self, commit_type: CommitType, enforce: bool = True
+    ) -> Optional[str]:
+        """Get the SHA for the commit type"""
+        commit_sha = (
+            self.before_change_sha
+            if commit_type is CommitType.BASE
+            else self.after_change_sha
+        )
+        if commit_sha is None and enforce:  # pragma: no cover
+            raise ValueError("Commit SHA is required")
+        return commit_sha
