@@ -14,6 +14,11 @@ from opendapi.adapters.git import (
     get_upstream_commit_sha,
 )
 from opendapi.cli.common import get_root_dir_validated
+from opendapi.cli.options import (
+    BASE_COMMIT_SHA_PARAM_NAME_WITH_OPTION,
+    HEAD_COMMIT_SHA_PARAM_NAME_WITH_OPTION,
+    HEAD_COMMIT_SHA_TIMESTAMP_PARAM_NAME_WITH_OPTION,
+)
 
 
 def construct_change_trigger_event(  # pylint: disable=too-many-locals
@@ -36,8 +41,18 @@ def construct_change_trigger_event(  # pylint: disable=too-many-locals
     repo_api_url = f"https://api.github.com/repos/{repo_owner}/{repo_name}"
     repo_html_url = f"https://github.com/{repo_owner}/{repo_name}"
 
-    after_change_sha = kwargs["buildkite_commit"]
-    if event_type == "push":
+    # can't really see a reason not to use the one from the event, but since
+    # we do the same for the timestamp might as well be consistent
+    after_change_sha = (
+        kwargs.get(HEAD_COMMIT_SHA_PARAM_NAME_WITH_OPTION.name)
+        or kwargs["buildkite_commit"]
+    )
+
+    if passed_in_base_commit_sha := kwargs.get(
+        BASE_COMMIT_SHA_PARAM_NAME_WITH_OPTION.name
+    ):
+        before_change_sha = passed_in_base_commit_sha
+    elif event_type == "push":
         before_change_sha = get_upstream_commit_sha(
             get_root_dir_validated(), after_change_sha, 1
         )
@@ -61,6 +76,10 @@ def construct_change_trigger_event(  # pylint: disable=too-many-locals
         else None
     )
 
+    after_change_sha_timestamp = kwargs.get(
+        HEAD_COMMIT_SHA_TIMESTAMP_PARAM_NAME_WITH_OPTION.name
+    ) or get_commit_timestamp_str(get_root_dir_validated(), after_change_sha)
+
     return ChangeTriggerEvent(
         where="github",
         event_type=event_type,
@@ -69,9 +88,7 @@ def construct_change_trigger_event(  # pylint: disable=too-many-locals
         repo_owner=repo_owner,
         before_change_sha=before_change_sha,
         after_change_sha=after_change_sha,
-        after_change_sha_timestamp=get_commit_timestamp_str(
-            get_root_dir_validated(), after_change_sha
-        ),
+        after_change_sha_timestamp=after_change_sha_timestamp,
         git_ref=git_ref,
         pull_request_number=pull_request_number,
         workspace=kwargs["buildkite_build_checkout_path"],
