@@ -11,6 +11,11 @@ from opendapi.adapters.git import (
     get_merge_base,
 )
 from opendapi.cli.common import get_root_dir_validated
+from opendapi.cli.options import (
+    BASE_COMMIT_SHA_PARAM_NAME_WITH_OPTION,
+    HEAD_COMMIT_SHA_PARAM_NAME_WITH_OPTION,
+    HEAD_COMMIT_SHA_TIMESTAMP_PARAM_NAME_WITH_OPTION,
+)
 from opendapi.utils import read_yaml_or_json
 
 
@@ -23,7 +28,9 @@ def construct_change_trigger_event(kwargs: dict) -> ChangeTriggerEvent:
     github_event_path = kwargs["github_event_path"]
     github_event = read_yaml_or_json(github_event_path)
 
-    after_change_sha = (
+    # can't really see a reason not to use the one from the event, but since
+    # we do the same for the timestamp might as well be consistent
+    after_change_sha = kwargs.get(HEAD_COMMIT_SHA_PARAM_NAME_WITH_OPTION.name) or (
         github_event["after"]
         if github_event_name == "push"
         else github_event["pull_request"]["head"]["sha"]
@@ -33,9 +40,14 @@ def construct_change_trigger_event(kwargs: dict) -> ChangeTriggerEvent:
         if github_event_name == "push"
         else github_event["pull_request"]["base"]["sha"]
     )
-    before_change_sha = get_merge_base(
-        get_root_dir_validated(), after_change_sha, base_branch_sha
-    )
+
+    before_change_sha = kwargs.get(
+        BASE_COMMIT_SHA_PARAM_NAME_WITH_OPTION.name
+    ) or get_merge_base(get_root_dir_validated(), after_change_sha, base_branch_sha)
+
+    after_change_sha_timestamp = kwargs.get(
+        HEAD_COMMIT_SHA_TIMESTAMP_PARAM_NAME_WITH_OPTION.name
+    ) or get_commit_timestamp_str(get_root_dir_validated(), after_change_sha)
 
     return ChangeTriggerEvent(
         where="github",
@@ -45,9 +57,7 @@ def construct_change_trigger_event(kwargs: dict) -> ChangeTriggerEvent:
         repo_owner=github_event["repository"]["owner"]["login"],
         before_change_sha=before_change_sha,
         after_change_sha=after_change_sha,
-        after_change_sha_timestamp=get_commit_timestamp_str(
-            get_root_dir_validated(), after_change_sha
-        ),
+        after_change_sha_timestamp=after_change_sha_timestamp,
         git_ref=(
             github_event["ref"]
             if github_event_name == "push"
@@ -67,6 +77,11 @@ def construct_change_trigger_event(kwargs: dict) -> ChangeTriggerEvent:
             if github_event_name == "pull_request"
             else None
         ),
+        original_pr_author=(
+            github_event["pull_request"].get("user", {}).get("login")
+            if github_event_name == "pull_request"
+            else None
+        ),
     )
 
 
