@@ -1,12 +1,12 @@
 """Teams validator module"""
 
-import functools
 from collections import Counter
 from typing import Dict, List, Tuple, Union
 
 from opendapi.defs import CATEGORIES_SUFFIX, OPENDAPI_SPEC_URL, OpenDAPIEntity
 from opendapi.validators.base import BaseValidator, ValidationError
 from opendapi.validators.defs import FileSet, MergeKeyCompositeIDParams
+from opendapi.weakref import weak_lru_cache
 
 
 class CategoriesValidator(BaseValidator):
@@ -26,10 +26,7 @@ class CategoriesValidator(BaseValidator):
         ]
     ] = [(["categories"], MergeKeyCompositeIDParams(required=[["urn"]]))]
 
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-
-    @functools.lru_cache()
+    @weak_lru_cache()
     def _get_file_state_category_urn_counts(self, fileset: FileSet) -> Counter:
         """Collect all the category urns"""
         return Counter(
