@@ -29,10 +29,9 @@ from opendapi.config import (
     get_project_path_from_full_path,
     is_model_in_allowlist,
 )
-from opendapi.defs import OPENDAPI_SPEC_URL
 from opendapi.models import OverrideConfig, PlaybookConfig
 from opendapi.utils import find_files_with_suffix
-from opendapi.validators.dapi.base import DapiValidator
+from opendapi.validators.dapi.base import DapiValidator, ORMIntegration
 from opendapi.validators.dapi.models import ProjectInfo
 
 
@@ -49,14 +48,7 @@ class Column:
         return {
             "name": self.column_name,
             "data_type": self.column_type,
-            "description": None,
-            "is_pii": None,
             "is_nullable": not self.column_options.get("null") == "false",
-            "access": "private",
-            "data_subjects_and_categories": [],
-            "sensitivity_level": None,
-            "is_personal_data": None,
-            "is_direct_identifier": None,
         }
 
 
@@ -105,47 +97,65 @@ class Table:  # pylint: disable=too-many-instance-attributes
             else None
         )
 
+    @property
+    def table_options_primary_keys(self) -> List[str]:
+        """Get the primary keys in the table options."""
+        if isinstance(self.table_options.get("primary_key"), str):
+            return [self.table_options.get("primary_key")]
+        if getattr(self.table_options.get("primary_key"), "asList", None):
+            return self.table_options["primary_key"].asList()
+        return []
+
     @property
     def columns(self) -> List[Column]:
         """Get the columns in the table."""
-        additional_columns = self.parsed_columns.copy()
         id_option = self.table_options.get("id", "true")
-        primary_key_option = self.table_options.get("primary_key", "id")
-        if id_option != "false":
-            if id_option == "true":
-                additional_columns.append(
-                    Column("bigint", "id", {"null": "false", "primary_key": "true"})
-                )
-            elif isinstance(primary_key_option, str):
-                additional_columns.append(
-                    Column(
-                        id_option,
-                        primary_key_option,
-                        {"null": "false", "primary_key": "true"},
-                    )
-                )
+        pk_col_name = None
+        pk_col_type = None
+        if primary_keys := self.table_options_primary_keys:
+            if len(primary_keys) == 1:
+                pk_col_name = primary_keys[0]
+                pk_col_type = "bigint" if id_option in {"true", "false"} else id_option
             else:
-                additional_columns.append(
-                    Column(id_option, "id", {"null": "false", "primary_key": "true"})
-                )
+                # for non-ID and composite primary keys,
+                # columns must be explicitly listed
+                pk_col_name = None
+                pk_col_type = None
+        elif id_option == "true":
+            pk_col_name = "id"
+            pk_col_type = "bigint"
+        elif id_option != "false":
+            pk_col_name = "id"
+            pk_col_type = id_option
+        else:
+            pk_col_name = None
+            pk_col_type = None
 
-        return additional_columns
+        columns_by_name = {col.column_name: col for col in self.parsed_columns}
+        if pk_col_name in columns_by_name:
+            columns_by_name[pk_col_name].column_options["primary_key"] = "true"
+            columns_by_name[pk_col_name].column_options["null"] = "false"
+        elif pk_col_name and pk_col_type:
+            columns_by_name[pk_col_name] = Column(
+                pk_col_type,
+                pk_col_name,
+                {"null": "false", "primary_key": "true"},
+            )
+
+        return list(columns_by_name.values())
 
     @property
     def primary_keys(self) -> List[str]:
         """Get the primary keys in the table."""
-        table_option_primary_key = self.table_options.get("primary_key")
-        if table_option_primary_key:
-            if isinstance(table_option_primary_key, str):
-                return [table_option_primary_key]
-            return list(table_option_primary_key)
+        if table_option_primary_keys := self.table_options_primary_keys:
+            return table_option_primary_keys
 
         # Case when the primary key is not specified in the table options
         columns_with_primary_key = []
         for column in self.columns:
             if column.column_options.get("primary_key") == "true":
                 columns_with_primary_key.append(column.column_name)
-        return list(columns_with_primary_key)
+        return columns_with_primary_key
 
     @property
     def project_name(self) -> str:
@@ -373,12 +383,12 @@ class ActiveRecordProjectInfo(ProjectInfo):
         return tables
 
 
-class ActiveRecordDapiValidator(DapiValidator):
+class ActiveRecordDapiValidator(DapiValidator[ActiveRecordProjectInfo]):
     """
     Validator class for DAPI files created for Rails ActiveRecord.
     """
 
-    INTEGRATION_NAME = "activerecord"
+    INTEGRATION_NAME = ORMIntegration.ACTIVERECORD
     SCHEMA_FILE_PATH_SUFFIX = "db/schema.rb"
 
     def _assert_schema_files_exist(self, projects: List[ActiveRecordProjectInfo]):
@@ -448,30 +458,26 @@ class ActiveRecordDapiValidator(DapiValidator):
             for table in project.tables:
                 if table.is_allowlisted:
                     dapi_location = project.construct_dapi_location(table.table_name)
-                    result[dapi_location] = {
-                        "schema": OPENDAPI_SPEC_URL.format(
-                            version=self.SPEC_VERSION,
-                            entity="dapi",
-                        ),
-                        "type": "entity",
-                        "urn": table.construct_urn(),
-                        "owner_team_urn": table.construct_team_urn()
-                        or self.get_owner_team_urn_for_table(table),
-                        "description": None,
-                        "datastores": table.construct_datastores(),
-                        "fields": [field.for_dapi() for field in table.columns],
-                        "primary_key": table.primary_keys,
-                        "context": {
-                            "service": table.project_name,
-                            "integration": "activerecord",
-                            "rel_model_path": os.path.relpath(
-                                project.artifact_full_path,
-                                os.path.dirname(dapi_location),
-                            ),
-                        },
-                        "privacy_requirements": {
-                            "dsr_access_endpoint": None,
-                            "dsr_deletion_endpoint": None,
-                        },
-                    }
+                    generated_fields = [field.for_dapi() for field in table.columns]
+                    dapi_urn = table.construct_urn()
+                    result[dapi_location] = (
+                        self.add_default_non_generated_schema_portions(
+                            {
+                                "urn": dapi_urn,
+                                "owner_team_urn": table.construct_team_urn()
+                                or self.get_owner_team_urn_for_table(table),
+                                "datastores": table.construct_datastores(),
+                                "fields": generated_fields,
+                                "primary_key": table.primary_keys,
+                                "context": {
+                                    "service": table.project_name,
+                                    "integration": "activerecord",
+                                    "rel_model_path": os.path.relpath(
+                                        project.artifact_full_path,
+                                        os.path.dirname(dapi_location),
+                                    ),
+                                },
+                            }
+                        )
+                    )
         return result
