@@ -6,7 +6,7 @@ import os
 import re
 from dataclasses import dataclass, field
 from enum import Enum
-from functools import cached_property, lru_cache
+from functools import cached_property
 from typing import Dict, List, Optional, Tuple
 
 from opendapi.config import (
@@ -16,15 +16,14 @@ from opendapi.config import (
     get_project_path_from_full_path,
     is_model_in_allowlist,
 )
-from opendapi.defs import OPENDAPI_SPEC_URL
-from opendapi.feature_flags import FeatureFlag, get_feature_flag
 from opendapi.logging import logger
 from opendapi.models import OverrideConfig
 from opendapi.utils import find_files_with_suffix, read_yaml_or_json
-from opendapi.validators.dapi.base import DapiValidator
-from opendapi.validators.dapi.mixins import DBTCloudMixin
+from opendapi.validators.dapi.base import DapiValidator, ORMIntegration
+from opendapi.validators.dapi.mixins.dbt_cloud import DBTCloudMixin, DBTCloudProject
 from opendapi.validators.dapi.models import ProjectInfo
 from opendapi.validators.defs import IntegrationType
+from opendapi.weakref import weak_lru_cache
 
 DBT_CONFIG_YML = "dbt_project.yml"
 DBT_ARTIFACTS_DIR = "target"
@@ -46,56 +45,59 @@ class ConstraintType(Enum):
     NOT_NULL = "not_null"
 
 
-def _use_tests_for_base_template() -> bool:
-    """Check if tests should be used for the base template"""
-    return get_feature_flag(FeatureFlag.DBT_WRITEBACK_ENABLED)
-
-
 @dataclass
 class DBTProjectInfo(ProjectInfo):
     """DBT project"""
 
     name: Optional[str] = None
     dbt_config: Dict = field(default_factory=dict)
+    cloud_manifest: Optional[Dict] = None
+    cloud_catalog: Optional[Dict] = None
+    cloud_production_catalog: Optional[Dict] = None
 
     @property
-    def manifest_filename(self) -> str:
-        """Get the manifest filename"""
-        return DBT_MANIFEST_JSON
-
-    @property
-    def catalog_filename(self) -> str:
-        """Get the catalog filename"""
-        return DBT_CATALOG_JSON
-
-    @property
-    def catalog_path(self) -> str:
+    def _catalog_path(self) -> str:
         """Get the catalog path"""
-        return os.path.join(self.artifact_full_path, self.catalog_filename)
+        return os.path.join(self.artifact_full_path, DBT_CATALOG_JSON)
 
     @property
-    def manifest_path(self) -> str:
+    def _manifest_path(self) -> str:
         """Get the manifest path"""
-        return os.path.join(self.artifact_full_path, self.manifest_filename)
+        return os.path.join(self.artifact_full_path, DBT_MANIFEST_JSON)
 
     @property
-    def production_catalog_path(self) -> str:
+    def _production_catalog_path(self) -> str:
         """Create the production catalog path"""
         return os.path.join(
-            os.path.dirname(self.catalog_path),
+            os.path.dirname(self._catalog_path),
             "prod",
-            os.path.basename(self.catalog_path),
+            os.path.basename(self._catalog_path),
         )
 
     @cached_property
     def manifest(self) -> Dict:
-        """Get the manifest"""
-        return read_yaml_or_json(self.manifest_path)
+        """Get the fallback manifest"""
+        return self.cloud_manifest or read_yaml_or_json(self._manifest_path)
+
+    @cached_property
+    def raw_catalog(self) -> Dict:
+        """Get the fallback catalog"""
+        return self.cloud_catalog or read_yaml_or_json(self._catalog_path)
+
+    @cached_property
+    def production_catalog(self) -> Optional[Dict]:
+        """Get the production catalog"""
+        try:
+            return self.cloud_production_catalog or read_yaml_or_json(
+                self._production_catalog_path
+            )
+        except FileNotFoundError:
+            return None
 
     @cached_property
     def catalog(self) -> Dict:
-        """Get the catalog"""
-        return read_yaml_or_json(self.catalog_path)
+        """Get the merged catalog"""
+        return DBTCloudProject.merge_catalogs(self.raw_catalog, self.production_catalog)
 
     def filter_dapis(self, dapis: Dict[str, Dict]) -> Dict[str, Dict]:
         """Get the owned DAPIs"""
@@ -105,6 +107,11 @@ class DBTProjectInfo(ProjectInfo):
             if dapi.get("context", {}).get("service") == self.name
         }
 
+    @property
+    def has_cloud_artifacts(self) -> bool:
+        """Check if the project has artifacts"""
+        return self.cloud_manifest and self.cloud_catalog
+
 
 @dataclass
 class DBTColumn:
@@ -132,9 +139,8 @@ class DBTColumn:
             for contract in self.manifest_info["constraints"]:
                 if contract.get("type") == ConstraintType.PRIMARY_KEY.value:
                     return True
-        if _use_tests_for_base_template():
-            if ColumnTestType.UNIQUE.value in self.tests_from_manifest:
-                return True
+        if ColumnTestType.UNIQUE.value in self.tests_from_manifest:
+            return True
         return False
 
     @property
@@ -144,16 +150,10 @@ class DBTColumn:
             for contract in self.manifest_info["constraints"]:
                 if contract.get("type") == ConstraintType.NOT_NULL.value:
                     return False
-        if _use_tests_for_base_template():
-            if ColumnTestType.NOT_NULL.value in self.tests_from_manifest:
-                return False
+        if ColumnTestType.NOT_NULL.value in self.tests_from_manifest:
+            return False
         return True
 
-    @property
-    def manifest_is_pii(self) -> bool:
-        """Check if the column contains PII"""
-        return self.manifest_info.get("meta", {}).get("contains_pii")
-
     def for_dapi(self) -> Dict:
         """Get the column for DAPI"""
         return {
@@ -161,12 +161,6 @@ class DBTColumn:
             "data_type": self.data_type,
             "description": self.manifest_description or None,
             "is_nullable": self.manifest_is_nullable,
-            "is_pii": self.manifest_is_pii,
-            "access": "private",
-            "data_subjects_and_categories": [],
-            "sensitivity_level": None,
-            "is_personal_data": None,
-            "is_direct_identifier": None,
         }
 
 
@@ -364,17 +358,30 @@ class DbtDapiValidator(DapiValidator[DBTProjectInfo], DBTCloudMixin):
 
     """
 
-    INTEGRATION_NAME = "dbt"
+    INTEGRATION_NAME = ORMIntegration.DBT
     INTEGRATION_TYPE = IntegrationType.DBT
 
-    def __init__(self, *, skip_generation, **kwargs):
+    def __init__(self, *, skip_generation: bool, **kwargs):
         self._skip_generation = skip_generation
         head_sha = os.environ.get("GITHUB_HEAD_SHA") or os.environ.get(
             "BUILDKITE_COMMIT"
         )
         self._dbt_commit_sha = kwargs.get("commit_sha") or head_sha
+        self._fast_fail = (
+            os.environ.get("DAPI_DBT_FAST_FAIL", "false").lower() == "true"
+        )
         super().__init__(**kwargs)
 
+    @property
+    def _github_pr_number(self) -> Optional[int]:
+        """Get the github pr number"""
+        return self.change_trigger_event.pull_request_number
+
+    @property
+    def _branch_name(self) -> Optional[str]:
+        """Get the branch name"""
+        return self.change_trigger_event.branch
+
     def get_all_dbt_config_files(self) -> List[str]:
         """Get all the dbt config files"""
         return find_files_with_suffix(
@@ -440,37 +447,40 @@ class DbtDapiValidator(DapiValidator[DBTProjectInfo], DBTCloudMixin):
 
     def _assert_necessary_files_exist(self, projects: List[DBTProjectInfo]) -> None:
         """Assert that the necessary files exist"""
-        if self._skip_generation:
+
+        # just in case, though this really should not be called
+        # in this instance
+        if self._skip_generation:  # pragma: no cover
             return
 
         errors = []
         for project in projects:
             try:
-                project.manifest
+                _ = project.manifest
             except FileNotFoundError:
                 errors.append(
-                    f"Manifest file not found for project {project.name} at {project.full_path}"
+                    f"Manifest file not found for project {project.name} "
+                    f"at {project.full_path}, or was not able to be synced "
+                    f"from dbt cloud"
                 )
 
             try:
-                project.catalog
+                _ = project.catalog
             except FileNotFoundError:
                 errors.append(
-                    f"Catalog file not found for project {project.name} at {project.full_path}"
+                    f"Catalog file not found for project {project.name} "
+                    f"at {project.full_path}, or was not able to be synced "
+                    f"from dbt cloud"
                 )
 
         if errors:
             raise FileNotFoundError("\n".join(errors))
 
-    def _cleanup_file_state(self, projects: List[DBTProjectInfo]) -> None:
-        """Cleanup the file state"""
-        self._cleanup_dbt_cloud_file_state(projects)
-
     def validate_projects(self, projects: List[DBTProjectInfo]):
         """Verify that all projects and their schema files exist"""
         self._assert_necessary_files_exist(projects)
 
-    @lru_cache(maxsize=8)
+    @weak_lru_cache(maxsize=8)
     def selected_projects(self, validate: bool = False) -> List[DBTProjectInfo]:
         """Get the selected projects"""
 
@@ -479,6 +489,10 @@ class DbtDapiValidator(DapiValidator[DBTProjectInfo], DBTCloudMixin):
         # ensure all the files are synced
         projects = super().selected_projects(validate=validate)
 
+        # we only download artifacts if we intend to generate
+        if self._skip_generation:
+            return projects
+
         # Try getting dbt files from dbt cloud or other sources. If integrated
         # with various other projects, the manifest and catalog files will be
         # synced from those sources.
@@ -488,10 +502,6 @@ class DbtDapiValidator(DapiValidator[DBTProjectInfo], DBTCloudMixin):
         # NOTE: loads stuff into memory
         self.validate_projects(projects)
 
-        # ensure that file state has not changed
-        # NOTE: must come last so state is loaded into memory
-        self._cleanup_file_state(projects)
-
         return projects
 
     def _get_dbt_models(self) -> List[DBTModel]:
@@ -552,10 +562,9 @@ class DbtDapiValidator(DapiValidator[DBTProjectInfo], DBTCloudMixin):
                     or os.environ.get("BUILDKITE_COMMIT"),
                 },
             )
-            projects = super().selected_projects()
             return {
                 fp: dapi
-                for project in projects
+                for project in self.selected_projects()
                 for fp, dapi in project.filter_dapis(self.original_file_state).items()
             }, {}
 
@@ -579,25 +588,20 @@ class DbtDapiValidator(DapiValidator[DBTProjectInfo], DBTCloudMixin):
                         os.path.dirname(dapi_location),
                     )
 
-                base_generated_files[dapi_location] = {
-                    "schema": OPENDAPI_SPEC_URL.format(
-                        version=self.SPEC_VERSION,
-                        entity="dapi",
-                    ),
-                    "urn": table.construct_urn(),
-                    "type": "entity",
-                    "description": table.manifest_table_description or None,
-                    "owner_team_urn": table.construct_owner_team_urn()
-                    or self.get_owner_team_urn_for_table(table),
-                    "datastores": table.construct_datastores(),
-                    "fields": [field.for_dapi() for field in table.columns],
-                    "primary_key": table.primary_keys,
-                    "context": context,
-                    "privacy_requirements": {
-                        "dsr_access_endpoint": None,
-                        "dsr_deletion_endpoint": None,
-                    },
-                }
+                base_generated_files[dapi_location] = (
+                    self.add_default_non_generated_schema_portions(
+                        {
+                            "urn": table.construct_urn(),
+                            "description": table.manifest_table_description or None,
+                            "owner_team_urn": table.construct_owner_team_urn()
+                            or self.get_owner_team_urn_for_table(table),
+                            "datastores": table.construct_datastores(),
+                            "fields": [field.for_dapi() for field in table.columns],
+                            "primary_key": table.primary_keys,
+                            "context": context,
+                        }
+                    )
+                )
                 generated_metadata[dapi_location] = {
                     "custom_schema": table.custom_schema,
                     "current_schema": table.schema,
@@ -611,3 +615,8 @@ class DbtDapiValidator(DapiValidator[DBTProjectInfo], DBTCloudMixin):
     def _get_base_generated_files(self) -> Dict[str, Dict]:
         """Get the base generated files"""
         return self._base_generated_files_and_generated_metadata[0]
+
+    @property
+    def _generate_skipped(self) -> bool:
+        """Check if generation is skipped"""
+        return self._skip_generation
