@@ -2,7 +2,7 @@
 
 from typing import Dict, Optional
 
-from opendapi.validators.dapi.base import BaseDapiValidator
+from opendapi.validators.dapi.base import BaseDapiValidator, ORMIntegration
 from opendapi.validators.defs import FileSet
 
 
@@ -12,7 +12,7 @@ class FallbackDapiValidator(BaseDapiValidator):
     just so that their Dapi files are known for downstream purposes, if necessary
     """
 
-    INTEGRATION_NAME = None
+    INTEGRATION_NAME = ORMIntegration.NO_ORM_FALLBACK
 
     def validate_existance_at(self, override: Optional[FileSet] = None):
         """
