@@ -27,6 +27,14 @@ class ProjectInfo:
         """Construct the location of the DAPI file within a project"""
         return f"{self.full_path}/dapis/{table_name}.dapi.yaml"
 
+    def filter_dapis(self, dapis: Dict[str, Dict]) -> Dict[str, Dict]:
+        """Get the owned DAPIs"""
+        return {
+            filepath: dapi
+            for filepath, dapi in dapis.items()
+            if filepath.startswith(self.full_path)
+        }
+
 
 @dataclass
 class PackageScopedProjectInfo(ProjectInfo):
