@@ -22,6 +22,7 @@ from pyparsing import (
     identbodychars,
     identchars,
     nestedExpr,
+    nums,
     quoted_string,
     remove_quotes,
     restOfLine,
@@ -34,10 +35,9 @@ from opendapi.config import (
     get_project_path_from_full_path,
     is_model_in_allowlist,
 )
-from opendapi.defs import OPENDAPI_SPEC_URL
 from opendapi.models import OverrideConfig, PlaybookConfig
 from opendapi.utils import find_files_with_suffix
-from opendapi.validators.dapi.base import DapiValidator
+from opendapi.validators.dapi.base import DapiValidator, ORMIntegration
 from opendapi.validators.dapi.models import ProjectInfo
 
 
@@ -61,14 +61,7 @@ class Column:
         return {
             "name": self.options.get("db_field") or self.name,
             "data_type": self._prisma_type_to_dapi_datatype(self.data_type),
-            "description": None,
-            "is_pii": None,
             "is_nullable": self.options.get("nullable", False),
-            "access": "private",
-            "data_subjects_and_categories": [],
-            "sensitivity_level": None,
-            "is_personal_data": None,
-            "is_direct_identifier": None,
         }
 
 
@@ -174,6 +167,8 @@ class PrismaProjectInfo(ProjectInfo):
         datetime = Keyword("DateTime")
         string = Keyword("String")
         unsupported = Keyword("Unsupported") + nestedExpr(ignore_expr=quoted_string())
+        # Define a numeric literal pattern
+        numeric_literal = Word(nums + ".")
 
         # Define a field type
         field_type = integer | datetime | string | boolean | unsupported | identifier
@@ -181,71 +176,104 @@ class PrismaProjectInfo(ProjectInfo):
         # Define various keywords
         comment = Suppress(Group(Suppress("//") + restOfLine))
 
-        default = Group(
-            AT
-            + Suppress(Keyword("default"))
-            + nestedExpr(ignore_expr=quoted_string() | Keyword("value:"))("value")
-        )("default")
+        # Define common patterns for decorators
+        name_option = Optional(Suppress(Keyword("name")) + Suppress(":")) + qtd_string(
+            "name_value"
+        )
+        map_option = Suppress(Keyword("map")) + Suppress(":") + qtd_string("map_value")
+        field_length_option = (
+            Keyword("length") + Suppress(":") + Word(nums)("length_value")
+        )
 
         # A primary key is defined as @id
-        id_attribute = Group(AT + Keyword("id"))("primary_key")
+        id_attribute = Group(AT + Keyword("id"))("is_primary_key")
 
         # A compound id is defined as @@id([field1, field2, ...])
         compound_id = Group(
             Keyword("@@id")
             + Suppress("([")
-            + delimitedList(identifier)
-            + Suppress("])")
-        )
+            + delimitedList(
+                identifier
+                + Optional(
+                    Suppress("(") + Suppress(field_length_option) + Suppress(")")
+                )
+            )("identifiers")
+            + Suppress("]")
+            + Optional(
+                Suppress(",")
+                + Suppress(delimitedList(name_option | map_option, delim=","))
+            )
+            + Suppress(")")
+        )("compound_id")
 
         # Unique field is defined as @unique or @@unique([field1, field2, ...])
-        unique = Group(AT + Keyword("unique"))
+        # or @@unique([field1, field2(length: 10), ...], name: "constraint_name", map: "db_name")
+        unique = Group(
+            Keyword("@unique")
+            + Optional(
+                Suppress("(")
+                + Suppress(delimitedList(name_option | map_option, delim=","))
+                + Suppress(")")
+            )
+        )("is_unique")
         compound_unique = Group(
             Keyword("@@unique")
             + Suppress("([")
-            + delimitedList(identifier)
-            + Suppress("])")
-        )
+            + delimitedList(
+                identifier
+                + Optional(
+                    Suppress("(") + Suppress(field_length_option) + Suppress(")")
+                )
+            )
+            + Suppress("]")
+            + Optional(
+                Suppress(",")
+                + Suppress(delimitedList(name_option | map_option, delim=","))
+            )
+            + Suppress(")")
+        )("compound_unique")
 
         # A @@map definition is use to map a model definition to a different table name
         table_mapping = Group(
-            Keyword("@@map")
-            + Suppress("(")
-            + Suppress(Optional(Keyword("name:")))
-            + qtd_string("table_name")
-            + Suppress(")")
-        )
+            Keyword("@@map") + Suppress("(") + name_option + Suppress(")")
+        )("table_map")
 
         # A @@schema definition is used to define the schema for a table
         schema_mapping = Group(
-            Keyword("@@schema")
-            + Suppress("(")
-            + Suppress(Optional(Keyword("name:")))
-            + qtd_string("schema_name")
-            + Suppress(")")
-        )
-
-        # Index field is defined as @@index([field1, field2, ...])
-        index = Suppress(Group(Keyword("@@index") + restOfLine))
+            Keyword("@@schema") + Suppress("(") + name_option + Suppress(")")
+        )("schema_map")
 
         # Map fields are used to map a model definition to an underlying database field
-        map_attribute = (
-            Keyword("@map")
-            + Suppress("(")
-            + Suppress(Optional(Keyword("name:")))
-            + qtd_string("db_field")
-            + Suppress(")")
+        map_attribute = Group(
+            AT + Keyword("map") + Suppress("(") + name_option + Suppress(")")
+        )("field_db_map")
+
+        # Default value definition
+        default = Group(
+            AT
+            + Suppress(Keyword("default"))
+            + nestedExpr(ignore_expr=quoted_string() | numeric_literal)("value")
+        )("default")
+
+        index = Suppress(Group(Keyword("@@index") + restOfLine))
+
+        generic_decorators = Suppress(
+            AT
+            + Word(alphanums + ".")
+            + Optional(nestedExpr(ignore_expr=quoted_string() | numeric_literal))
         )
 
-        # This is used only in mongodb schema definitions
-        # db_object_id = Suppress(Keyword("@db.ObjectId"))
-
-        # Catch generic decorators
-        generic_decorators = Suppress(AT + Word(alphanums + "."))
+        # Ignore field definition
+        ignore_attribute = Group(AT + Keyword("ignore"))("is_ignored")
 
         decorators = Group(
             ZeroOrMore(
-                default | id_attribute | unique | map_attribute | generic_decorators
+                default
+                | id_attribute
+                | unique
+                | map_attribute
+                | ignore_attribute
+                | generic_decorators
             )
         )
         relationship = Suppress(
@@ -271,6 +299,9 @@ class PrismaProjectInfo(ProjectInfo):
             + Optional(comment)
         )("field")
 
+        # Ignore model definition
+        ignore_model = Group(Keyword("@@ignore"))("is_ignored")
+
         # A table is defined as
         # model table_name {
         #   field1 field_type @default(value) @id @unique  // comment
@@ -292,7 +323,7 @@ class PrismaProjectInfo(ProjectInfo):
                     | compound_id
                     | compound_unique
                     | index
-                    | Suppress("@@ignore")
+                    | ignore_model
                     | table_mapping
                     | schema_mapping
                 )
@@ -302,7 +333,12 @@ class PrismaProjectInfo(ProjectInfo):
 
         tables = []
         parsed_models = set()
+        ignored_models = set()
         for table in models.searchString(schema):
+            if table.info.is_ignored:
+                ignored_models.add(table.table_name)
+                continue
+
             primary_keys = []
             columns = []
             table_name = table.table_name
@@ -311,7 +347,7 @@ class PrismaProjectInfo(ProjectInfo):
             field_map = {}
 
             for row in table.info:
-                if row.name:
+                if row.name and not row.info.is_ignored:
                     # This is a parsed column
                     column = row
                     default_value = (
@@ -320,8 +356,8 @@ class PrismaProjectInfo(ProjectInfo):
                         else None
                     )
                     db_field = (
-                        column.info.db_field
-                        if "db_field" in column.info
+                        column.info.field_db_map.name_value
+                        if "field_db_map" in column.info
                         else column.name
                     )
                     field_map[column.name] = db_field
@@ -341,19 +377,19 @@ class PrismaProjectInfo(ProjectInfo):
                         )
                     )
 
-                    if column.info.primary_key:
+                    if column.info.is_primary_key:
                         primary_keys.append(column.name)
 
                 elif row[0] == "@@id":
                     # This is a parsed compound primary key
-                    primary_keys = row[1:]
+                    primary_keys = row.identifiers.as_list()
 
                 elif row[0] == "@@map":
                     # Remove the quotes and store the table name
-                    table_name = row[1]
+                    table_name = row.name_value
 
                 elif row[0] == "@@schema":
-                    schema_name = row[1]
+                    schema_name = row.name_value
 
             primary_keys = [field_map[pk] for pk in primary_keys]
 
@@ -380,13 +416,12 @@ class PrismaProjectInfo(ProjectInfo):
                 )
             )
 
-        missed_models = expected_models - parsed_models
+        missed_models = expected_models - parsed_models - ignored_models
         if missed_models:
             raise RuntimeError(
                 f"Missed parsing the following models: {', '.join(missed_models)} "
                 + f"from {self.artifact_full_path}"
             )
-
         return tables
 
 
@@ -395,7 +430,7 @@ class PrismaDapiValidator(DapiValidator[PrismaProjectInfo]):
     Validator class for DAPI files created for Prisma schemas
     """
 
-    INTEGRATION_NAME = "prisma"
+    INTEGRATION_NAME = ORMIntegration.PRISMA
 
     SCHEMA_FILE_PATH_SUFFIX = ".prisma"
     # This is used by the npm prisma-multischema package. We will exclude the
@@ -476,32 +511,26 @@ class PrismaDapiValidator(DapiValidator[PrismaProjectInfo]):
             for table in project.tables:
                 if table.is_allowlisted:
                     dapi_location = project.construct_dapi_location(table.table_name)
-                    result[dapi_location] = {
-                        "schema": OPENDAPI_SPEC_URL.format(
-                            version=self.SPEC_VERSION,
-                            entity="dapi",
-                        ),
-                        "type": "entity",
-                        "urn": table.construct_urn(),
-                        "owner_team_urn": table.construct_team_urn()
-                        or self.get_owner_team_urn_for_table(table),
-                        "description": None,
-                        "datastores": table.construct_datastores(),
-                        "fields": [field.for_dapi() for field in table.columns],
-                        "primary_key": table.primary_keys,
-                        "context": {
-                            "service": table.project_name,
-                            "integration": "prisma",
-                            "rel_model_path": os.path.relpath(
-                                project.artifact_full_path,
-                                os.path.dirname(dapi_location),
-                            ),
-                        },
-                        "privacy_requirements": {
-                            "dsr_access_endpoint": None,
-                            "dsr_deletion_endpoint": None,
-                        },
-                    }
+                    result[dapi_location] = (
+                        self.add_default_non_generated_schema_portions(
+                            {
+                                "urn": table.construct_urn(),
+                                "owner_team_urn": table.construct_team_urn()
+                                or self.get_owner_team_urn_for_table(table),
+                                "datastores": table.construct_datastores(),
+                                "fields": [field.for_dapi() for field in table.columns],
+                                "primary_key": table.primary_keys,
+                                "context": {
+                                    "service": table.project_name,
+                                    "integration": "prisma",
+                                    "rel_model_path": os.path.relpath(
+                                        project.artifact_full_path,
+                                        os.path.dirname(dapi_location),
+                                    ),
+                                },
+                            }
+                        )
+                    )
         return result
 
 
