@@ -37,8 +37,7 @@ from opendapi.config import (
     construct_owner_team_urn_from_playbooks,
     is_model_in_allowlist,
 )
-from opendapi.defs import OPENDAPI_SPEC_URL
-from opendapi.validators.dapi.base import PackageScopedDapiValidatorBase
+from opendapi.validators.dapi.base import ORMIntegration, PackageScopedDapiValidatorBase
 from opendapi.validators.dapi.models import PackageScopedProjectInfo
 
 # Memoize the parser element to improve performance
@@ -178,14 +177,7 @@ class Column:
         return {
             "name": self.name,
             "data_type": self.type,
-            "description": None,
-            "is_pii": None,
             "is_nullable": self.is_nullable,
-            "access": "private",
-            "data_subjects_and_categories": [],
-            "sensitivity_level": None,
-            "is_personal_data": None,
-            "is_direct_identifier": None,
         }
 
     def is_foreign_key(self) -> bool:
@@ -521,7 +513,7 @@ class TypeOrmDapiValidator(PackageScopedDapiValidatorBase):
         - Default exports out a module if it is imported with a different name in another
     """
 
-    INTEGRATION_NAME = "typeorm"
+    INTEGRATION_NAME = ORMIntegration.TYPEORM
     LOOKUP_FILE_SUFFIXES = [".ts"]
 
     # Only parse files if they contain any of these keywords, to limit the number of files parsed
@@ -877,30 +869,24 @@ class TypeOrmDapiValidator(PackageScopedDapiValidatorBase):
 
             dapi_location = table.project.construct_dapi_location(table.name)
 
-            result[dapi_location] = {
-                "schema": OPENDAPI_SPEC_URL.format(
-                    version=self.SPEC_VERSION,
-                    entity="dapi",
-                ),
-                "urn": table.construct_urn(),
-                "owner_team_urn": table.construct_team_urn(),
-                "description": None,
-                "type": "entity",
-                "datastores": table.construct_datastores(),
-                "fields": [
-                    field.for_dapi()
-                    for field in table.compile_columns(processed_tables)
-                ],
-                "primary_key": [c.name for c in table.primary_keys],
-                "context": {
-                    "service": table.project_name,
-                    "integration": "typeorm",
-                    "rel_model_path": os.path.relpath(table.filename, dapi_location),
-                },
-                "privacy_requirements": {
-                    "dsr_access_endpoint": None,
-                    "dsr_deletion_endpoint": None,
-                },
-            }
+            result[dapi_location] = self.add_default_non_generated_schema_portions(
+                {
+                    "urn": table.construct_urn(),
+                    "owner_team_urn": table.construct_team_urn(),
+                    "datastores": table.construct_datastores(),
+                    "fields": [
+                        field.for_dapi()
+                        for field in table.compile_columns(processed_tables)
+                    ],
+                    "primary_key": [c.name for c in table.primary_keys],
+                    "context": {
+                        "service": table.project_name,
+                        "integration": "typeorm",
+                        "rel_model_path": os.path.relpath(
+                            table.filename, dapi_location
+                        ),
+                    },
+                }
+            )
 
         return result
