@@ -57,6 +57,7 @@ class CollectedFile:  # pylint: disable=too-many-instance-attributes
     entity: OpenDAPIEntity
     root_dir: str
     additional_metadata_from_generated: Optional[Dict] = None
+    generate_skipped: bool = False
 
     @property
     def server_filepath(self) -> str:
@@ -74,6 +75,7 @@ class CollectedFile:  # pylint: disable=too-many-instance-attributes
             "commit_sha": self.commit_sha,
             "entity": self.entity.value,
             "additional_metadata_from_generated": self.additional_metadata_from_generated,
+            "generate_skipped": self.generate_skipped,
         }
 
     @property
@@ -88,6 +90,7 @@ class CollectedFile:  # pylint: disable=too-many-instance-attributes
             "entity": self.entity.value,
             "root_dir": self.root_dir,
             "additional_metadata_from_generated": self.additional_metadata_from_generated,
+            "generate_skipped": self.generate_skipped,
         }
 
     @classmethod
@@ -104,6 +107,79 @@ class CollectedFile:  # pylint: disable=too-many-instance-attributes
             additional_metadata_from_generated=dict_.get(
                 "additional_metadata_from_generated"
             ),
+            generate_skipped=dict_.get("generate_skipped", False),
+        )
+
+    def reconcile(  # pylint: disable=too-many-branches
+        self,
+        other: Optional[CollectedFile],
+    ) -> CollectedFile:
+        """
+        Reconcile two CollectedFiles, choosing actually generated values.
+        This is useful for multi-runtime setups, where some runtimes may
+        run and be skipped, or run the same generation multiple times.
+        """
+        if not other:
+            return self
+
+        if self.filepath != other.filepath:
+            raise ValueError("Cannot reconcile files with different filepaths")
+
+        if self.entity is not other.entity:
+            raise ValueError("Cannot reconcile files with different entities")
+
+        if self.commit_sha != other.commit_sha:
+            raise ValueError("Cannot reconcile files with different commit SHAs")
+
+        if self.original != other.original:
+            raise ValueError("Cannot reconcile files with different original contents")
+
+        # choose the appropriate generated
+        if self.generate_skipped == other.generate_skipped:
+            if (
+                self.generated != other.generated
+                or self.additional_metadata_from_generated
+                != other.additional_metadata_from_generated
+            ):
+                raise ValueError(
+                    "Cannot reconcile files with different generated contents or metadatas"
+                )
+            generate_skipped = self.generate_skipped
+            generated = self.generated
+            additional_metadata_from_generated = self.additional_metadata_from_generated
+        elif self.generate_skipped:
+            generate_skipped = other.generate_skipped
+            generated = other.generated
+            additional_metadata_from_generated = (
+                other.additional_metadata_from_generated
+            )
+        else:
+            generate_skipped = self.generate_skipped
+            generated = self.generated
+            additional_metadata_from_generated = self.additional_metadata_from_generated
+
+        # do the same for merged
+        if self.generate_skipped == other.generate_skipped:
+            if self.merged != other.merged:
+                raise ValueError(
+                    "Cannot reconcile files with different merged contents"
+                )
+            merged = self.merged
+        elif self.generate_skipped:
+            merged = other.merged
+        else:
+            merged = self.merged
+
+        return CollectedFile(
+            original=self.original,
+            generated=generated,
+            merged=merged,
+            filepath=self.filepath,
+            commit_sha=self.commit_sha,
+            entity=self.entity,
+            additional_metadata_from_generated=additional_metadata_from_generated,
+            generate_skipped=generate_skipped,
+            root_dir=self.root_dir,
         )
 
 
