@@ -1,12 +1,12 @@
 """Teams validator module"""
 
-import functools
 from collections import Counter
 from typing import Dict, List, Tuple, Union
 
 from opendapi.defs import OPENDAPI_SPEC_URL, PURPOSES_SUFFIX, OpenDAPIEntity
 from opendapi.validators.base import BaseValidator, ValidationError
 from opendapi.validators.defs import FileSet, MergeKeyCompositeIDParams
+from opendapi.weakref import weak_lru_cache
 
 
 class PurposesValidator(BaseValidator):
@@ -26,10 +26,7 @@ class PurposesValidator(BaseValidator):
         ]
     ] = [(["purposes"], MergeKeyCompositeIDParams(required=[["urn"]]))]
 
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-
-    @functools.lru_cache()
+    @weak_lru_cache()
     def _get_file_state_purpose_urn_counts(self, fileset: FileSet) -> Counter:
         """Collect all the purposes urns"""
         return Counter(
